/* This example sketch illustrates how to use the Azatrax library
 * functions to communicate with the stopwatches (timers) on an
 * RIR4 shield.
 * Provides examples of enabling a stopwatch, reading the time 
 * and direction, and resetting a stopwatch.
 * 
 * Written by John Parsons for public use.  No warranty.
 * revised:
 * 10 May 2018 Initial release.
 * 10 Mar 2023 File name changed to remove '-' which is no longer allowed by Ard
uino IDE.
 */

#include <Azatrax.h>    // enables access to the Azatrax library functions

// stopwatch #1 is referred to as stopwatch A in the RIR4 user's guide.
// stopwatch #2 is referred to as stopwatch B in the RIR4 user's guide.
const byte stopwA = 1;
const byte stopwB = 2;

Azatrax RIR4(0x38); // Create RIR4 as an instance of class Azatrax.
                    // This is the RIR4 circuit with I2C address set to 0x38.
// If more than one RIR4 is being used, give each one a unique name
// and set the I2C address selection switches to unique values for each RIR4.
// Initialize each RIR4 (create an instance of class Azatrax for each circuit).
// For example,
//   Azatrax RIR41(0x30); // first RIR4, addr switch set for 0x30
//   Azatrax RIR42(0x32); // second RIR4, addr switch set for 0x32
//   Azatrax RIR43(0x34); // third RIR4, addr switch set for 0x34
//
// Then, where you find RIR4 in the following code, substitute RIR41, RIR42 or
// RIR43 in place of RIR4.

// Prototype function declaration
// This function is found at the end of this sketch.  This declaration
// informs the compiler of the function's existence.
int keyedNumericInput(void);

void setup() {
    Serial.begin(9600);   // to use the serial monitor
}

void loop() {
    static byte i2cAddr;
    static byte detNum;
    static byte devData;
    static byte newThr;
    static char rxChar;

    Serial.println();
    // Read the contents of RIR4's internal register 0x0A (stopwatch state bitmap).
    // The upper bits should all be 0.
    devData = RIR4.getDetData(0x0A);
    if(0xFF == devData) {
        // If the returned byte is 0xFF then the communication did
        // not succeed, probably due to no RIR4 at the given I2C address.
        i2cAddr = (byte)RIR4.devAddr;   // Get the address that was used when this
                        // RIR4 was initialized at the top of this sketch.
                        // Convert it to byte format.
        Serial.print("Device at address 0x");
        Serial.print(i2cAddr, HEX);     // print in hexadecimal characters
        Serial.println(" did not respond as expected.");
    }
    else {
        // A good reply was received...
        Serial.print("Stopwatch state bitmap: ");
        Serial.println(devData, BIN);   // print in binary characters
        // A bit value of 1 indicates the stopwatch is running, 0 indicates stopped.

        // Using the boolean functions:
        Serial.print("   Stopwatch B:");
        if(true == RIR4.stopwatchRunning(stopwB)) {   // this function returns 'true'
            Serial.print("running");       // when the specified stopwatch is running
        }
        else {
            Serial.print("stopped");
        }
        Serial.print("   Stopwatch A:");
        if(true == RIR4.stopwatchStopped(stopwA)) {   // this function returns 'true'
            Serial.println("stopped");       // when the specified stopwatch is running
        }
        else {
            Serial.println("running");
        }
    
        // Get and display the direction of the object that triggered each stopwatch.
        // Stopwatch A is associated with sensors #1 & #2.
        // Stopwatch B is associated with sensors #3 & #4.
        // After being reset, a stopwatch can be started by an event (change of state)
        // at either sensor, and will stop by an equivalent event at its other sensor.
        // A stopwatch will stop if its time reaches 255 minutes.
        Serial.print("   B direction: ");
        devData = RIR4.stopwatchDir(stopwB);
        switch (devData) {
            case 0:
                Serial.print(" (reset)      ");
                break;
            case 3:
                Serial.print(" from #3 to #4");
                break;
            case 4:
                Serial.print(" from #4 to #3");
                break;
            default:
                Serial.print(" unexpected value ");
                Serial.print(devData);
        }        
        Serial.print("   A direction: ");
        devData = RIR4.stopwatchDir(stopwA);
        switch (devData) {
            case 0:
                Serial.print(" (reset)      ");
                break;
            case 1:
                Serial.print(" from #1 to #2");
                break;
            case 2:
                Serial.print(" from #2 to #1");
                break;
            default:
                Serial.print(" unexpected value ");
                Serial.print(devData);
        }
        Serial.println();
    
        // Get and display the minutes, seconds and 1/100th seconds for
        // each stopwatch.
        Serial.print("Stopwatch time B: ");
        devData = RIR4.stopwatchMin(stopwB);
        Serial.print(devData);
        Serial.print(":");      // separator between minutes and seconds
        devData = RIR4.stopwatchSec(stopwB);
        Serial.print(devData);
        Serial.print(".");      // decimal pt between seconds and 1/100th seconds
        devData = RIR4.stopwatch100(stopwB);
        Serial.print(devData);
        
        Serial.print("   A: ");
        devData = RIR4.stopwatchMin(stopwA);
        Serial.print(devData);
        Serial.print(":");      // separator between minutes and seconds
        devData = RIR4.stopwatchSec(stopwA);
        Serial.print(devData);
        Serial.print(".");      // decimal pt between seconds and 1/100th seconds
        devData = RIR4.stopwatch100(stopwA);
        Serial.println(devData);
    }
    // Give the user an opportunity to reset a stopwatch.
    Serial.println();
    Serial.println("To reset/enable a stopwatch, enter its letter (A or B) above,");
    Serial.println("or, enter any other character to continue...");
    while(0 == Serial.available()) {
    // Wait here until a character is entered on the serial monitor
    }
    while(0 < Serial.available()) {
        rxChar = Serial.read();
        switch (rxChar) {
          // allow upper and lower case to be accepted
          case 'a':
          case 'A':
              Serial.println("resetting stopwatch A");
              RIR4.stopwatchReset(stopwA);
              break;
          case 'b':
          case 'B':
              Serial.println("resetting stopwatch B");
              RIR4.stopwatchReset(stopwB);
              break;
              default:
                ;   // for other characters, just continue
        }
    delay(10);  // Allow next character to enter the buffer.
                // Without this delay, the loop sometimes exits before
                // the entire input string is processed.
    }
}

/***********************************
 * keyedNumericInput
 * This routine accepts serial input, such as from the serial monitor,
 * decodes the ASCII characters and returns an integer.
 *
 ***********************************/
int keyedNumericInput()
{
    String inString;    // string to hold the ASCII input
    char inChar;        // holds the next character from input

    inString = "";
    while(Serial.available() == 0) {
        // Wait for user input
    }
    while(Serial.available() > 0) {
        // Build a string out of the entered characters that are numeric
        // digits (between '0' and '9').
        inChar = (char)Serial.read();
        if(('0' <= inChar) && (inChar <= '9')) {
            inString.concat(inChar);   // appends inChar to the end of inString
        }
        delay(10);  // Allow next character to enter the buffer.
                    // Without this delay, the loop sometimes exits before
                    // the entire input string is processed.
    }
    return inString.toInt();
}

